@@ -1,5 +1,5 @@
 /*
- * $Id: file_cache.h,v 1.4 2003/07/21 16:31:17 hipnod Exp $
+ * $Id: file_cache.h,v 1.6 2004/03/24 06:28:16 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -14,8 +14,10 @@
  * General Public License for more details.
  */
 
-#ifndef __GIFT_FILE_CACHE_H
-#define __GIFT_FILE_CACHE_H
+#ifndef GIFT_GT_FILE_CACHE_H_
+#define GIFT_GT_FILE_CACHE_H_
+
+/*****************************************************************************/
 
 typedef struct _file_cache
 {
@@ -24,14 +26,21 @@ typedef struct _file_cache
 	char    *file;
 } FileCache;
 
-FileCache    *file_cache_new    (char *file);
+/*****************************************************************************/
+
+FileCache    *file_cache_new    (const char *file);
 void          file_cache_free   (FileCache *cache);
 int           file_cache_load   (FileCache *cache);
 void          file_cache_flush  (FileCache *cache);
 BOOL          file_cache_sync   (FileCache *cache);
 
-char         *file_cache_lookup (FileCache *cache, char *key);
-void          file_cache_insert (FileCache *cache, char *key, char *value);
-void          file_cache_remove (FileCache *cache, char *key);
+/*****************************************************************************/
 
-#endif /* __GIFT_FILE_CACHE_H */
+char         *file_cache_lookup (FileCache *cache, const char *key);
+void          file_cache_insert (FileCache *cache, const char *key,
+                                 const char *value);
+void          file_cache_remove (FileCache *cache, const char *key);
+
+/*****************************************************************************/
+
+#endif /* GIFT_GT_FILE_CACHE_H_ */
