@@ -1,5 +1,5 @@
 /*
- * $Id: gt_accept.h,v 1.6 2003/09/17 17:44:11 hipnod Exp $
+ * $Id: gt_accept.h,v 1.10 2005/01/04 15:03:40 mkern Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -14,22 +14,31 @@
  * General Public License for more details.
  */
 
-#ifndef __GT_ACCEPT_H__
-#define __GT_ACCEPT_H__
+#ifndef GIFT_GT_ACCEPT_H_
+#define GIFT_GT_ACCEPT_H_
 
 /*****************************************************************************/
 
 void  gnutella_handle_incoming  (int fd, input_id, TCPC *c);
-void  gnutella_determine_method (int fd, input_id, TCPC *c);
 
 BOOL  gnutella_will_deflate     (struct gt_node *node);
-void  gnutella_mark_completed   (struct gt_node *node);
+void  gnutella_mark_compression (struct gt_node *node);
 
-int   gnutella_auth_connection  (TCPC *c);
+BOOL  gnutella_auth_connection  (TCPC *c);
 
-void  http_headers_parse       (char *headers, Dataset **dataset);
-int   http_headers_terminated  (char *data, size_t len);
+void  gt_http_header_parse      (char *headers, Dataset **dataset);
+BOOL  gt_http_header_terminated (char *data, size_t len);
+
+void  gt_handshake_dispatch_incoming (int fd, input_id id, TCPC *c);
 
 /*****************************************************************************/
 
-#endif /*  __GT_ACCEPT_H__ */
+/*
+ * Returns TRUE if an arbitrary limit on the buffer underlying the FDBuf has
+ * been reached.
+ */
+BOOL gt_fdbuf_full              (FDBuf *buf);
+
+/*****************************************************************************/
+
+#endif /* GIFT_GT_ACCEPT_H_ */
