@@ -1,5 +1,5 @@
 /*
- * $Id: gt_conf.h,v 1.1 2003/06/07 05:24:14 hipnod Exp $
+ * $Id: gt_conf.h,v 1.2 2004/03/24 06:27:40 hipnod Exp $
  *
  * Copyright (C) 2003 giFT project (gift.sourceforge.net)
  *
@@ -14,17 +14,17 @@
  * General Public License for more details.
  */
 
-#ifndef __GT_CONF_H__
-#define __GT_CONF_H__
+#ifndef GIFT_GT_CONF_H_
+#define GIFT_GT_CONF_H_
 
 /*****************************************************************************/
 
-char   *gt_config_get_str   (char *key);
-int     gt_config_get_int   (char *key);
+char   *gt_config_get_str   (const char *key);
+int     gt_config_get_int   (const char *key);
 
 /*****************************************************************************/
 
-BOOL    gt_config_load_file (char *path, BOOL update, BOOL force);
+BOOL    gt_config_load_file (const char *path, BOOL update, BOOL force);
 
 /*****************************************************************************/
 
@@ -33,4 +33,4 @@ void    gt_config_cleanup   (void);
 
 /*****************************************************************************/
 
-#endif /* __GT_CONF_H__ */
+#endif /* GIFT_GT_CONF_H_ */
