@@ -1,5 +1,5 @@
 /*
- * $Id: gt_connect.h,v 1.8 2003/09/17 17:44:11 hipnod Exp $
+ * $Id: gt_connect.h,v 1.9 2004/03/24 06:21:13 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -14,8 +14,8 @@
  * General Public License for more details.
  */
 
-#ifndef __GT_CONNECT_H__
-#define __GT_CONNECT_H__
+#ifndef GIFT_GT_CONNECT_H_
+#define GIFT_GT_CONNECT_H_
 
 /*****************************************************************************/
 
@@ -25,9 +25,9 @@
 
 struct gt_node;
 
-int   gnutella_send_connection_headers (TCPC *c, char *header);
+BOOL  gnutella_send_connection_headers (TCPC *c, const char *header);
 void  gnutella_start_connection        (int fd, input_id id, TCPC *c);
-int   gnutella_parse_response_headers  (char *response, Dataset **headers);
+BOOL  gnutella_parse_response_headers  (char *response, Dataset **headers);
 void  gnutella_set_handshake_timeout   (TCPC *c, time_t time);
 
 void  gt_connect_test     (struct gt_node *node, in_port_t port);
@@ -35,4 +35,4 @@ int   gt_connect          (struct gt_node *node);
 
 /*****************************************************************************/
 
-#endif /* __GT_CONNECT_H__ */
+#endif /* GIFT_GT_CONNECT_H_ */
