@@ -1,5 +1,5 @@
 /*
- * $Id: gt_guid.h,v 1.8 2003/12/11 07:25:59 hipnod Exp $
+ * $Id: gt_guid.h,v 1.10 2004/03/05 17:49:40 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -14,8 +14,8 @@
  * General Public License for more details.
  */
 
-#ifndef __GT_GUID_H__
-#define __GT_GUID_H__
+#ifndef GIFT_GT_GUID_H_
+#define GIFT_GT_GUID_H_
 
 /*****************************************************************************/
 
@@ -27,14 +27,22 @@ typedef uint8_t gt_guid_t;
 
 /*****************************************************************************/
 
-gt_guid_t  *gt_guid_new      (void);
-void        gt_guid_init     (gt_guid_t *guid);
-int         gt_guid_cmp      (gt_guid_t *a, gt_guid_t *b);
-char       *gt_guid_str      (gt_guid_t *guid);
-gt_guid_t  *gt_guid_dup      (const gt_guid_t *guid);
-gt_guid_t  *gt_guid_bin      (const char *guid_ascii);
-BOOL        gt_guid_is_empty (gt_guid_t *guid);
+gt_guid_t  *gt_guid_new          (void);
+void        gt_guid_init         (gt_guid_t *guid);
+int         gt_guid_cmp          (const gt_guid_t *a, const gt_guid_t *b);
+char       *gt_guid_str          (const gt_guid_t *guid);
+gt_guid_t  *gt_guid_dup          (const gt_guid_t *guid);
+gt_guid_t  *gt_guid_bin          (const char *guid_ascii);
 
 /*****************************************************************************/
 
-#endif /* __GT_GUID_H__ */
+BOOL        gt_guid_is_empty     (const gt_guid_t *guid);
+
+/*****************************************************************************/
+
+void        gt_guid_self_init    (void);
+void        gt_guid_self_cleanup (void);
+
+/*****************************************************************************/
+
+#endif /* GIFT_GT_GUID_H_ */
