@@ -1,5 +1,5 @@
 /*
- * $Id: gt_http_client.c,v 1.46 2003/12/23 18:06:34 jasta Exp $
+ * $Id: gt_http_client.c,v 1.54 2005/01/04 15:03:40 mkern Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -15,6 +15,7 @@
  */
 
 #include "gt_gnutella.h"
+#include "gt_version.h"
 
 #include "gt_xfer_obj.h"
 #include "gt_xfer.h"
@@ -23,21 +24,14 @@
 #include "gt_http_server.h"
 
 #include "gt_accept.h"
-#include "gt_version.h"
 
 #include "transfer/source.h"
 
 /*****************************************************************************/
 
-/*
- * LOTS OF DOCUMENTATION BADLY NEEDED HERE
- */
-
-/*****************************************************************************/
-
-/* prototyping this function effectively provides the non-blocking
- * flow of the program and helps to self-document this file */
-static void get_server_reply     (int fd, input_id id, TCPC *c);
+/* prototyping this function effectively provides the non-blocking flow of the
+ * program and helps to self-document this file */
+static void get_server_reply     (int fd, input_id id, GtTransfer *xfer);
 
 /*****************************************************************************/
 /* CLIENT HELPERS */
@@ -108,7 +102,7 @@ static BOOL parse_server_reply (GtTransfer *xfer, TCPC *c, char *reply)
 	code    = ATOI (string_sep (&response, " ")); /* shift past 200 */
 
 	/* parse the rest of the key/value fields */
-	http_headers_parse (reply, &xfer->header);
+	gt_http_header_parse (reply, &xfer->header);
 
 	xfer->code    = code;
 	xfer->version = STRDUP (version);
@@ -140,27 +134,29 @@ void gt_http_client_get (Chunk *chunk, GtTransfer *xfer)
 
 	xfer->command = STRDUP ("GET");
 
-	if (!(c = gt_http_connection_open (xfer->ip, xfer->port,
-	                                   GT_TRANSFER_DOWNLOAD)))
+	if (!(c = gt_http_connection_open (GT_TRANSFER_DOWNLOAD, xfer->ip,
+	                                   xfer->port)))
 	{
 		gt_transfer_close (xfer, TRUE);
 		return;
 	}
 
-	/* pass along the chunk data with this connection.  the chunk data
-	 * passes along the GtTransfer data */
-	gt_transfer_ref (c, chunk, xfer);
+	/* pass along the connection with the xfer */
+	gt_transfer_set_tcpc (xfer, c);
+	assert (xfer->chunk == chunk);
+	assert (chunk->udata == xfer);
+
 	gt_transfer_status (xfer, SOURCE_WAITING, "Connecting");
 
 	/* be a little more aggressive timing out HTTP connections (TIMEOUT_DEF /
 	 * 2 + 5), so that useless sources don't occupy Chunks so often */
-	input_add (c->fd, c, INPUT_WRITE,
+	input_add (c->fd, xfer, INPUT_WRITE,
 	           (InputCallback)gt_http_client_start, TIMEOUT_DEF / 2 + 5);
 }
 
 static int client_get_request (GtTransfer *xfer)
 {
-	TCPC  *c = NULL;
+	TCPC  *c;
 	char   host[128];
 	char   range_hdr[64];
 	int    ret;
@@ -168,7 +164,7 @@ static int client_get_request (GtTransfer *xfer)
 	if (!xfer)
 		return FALSE;
 
-	gt_transfer_unref (&c, NULL, &xfer);
+	c = gt_transfer_get_tcpc (xfer);
 
 	snprintf (range_hdr, sizeof (range_hdr) - 1, "bytes=%i-%i",
 	          (int) xfer->start, (int) xfer->stop - 1);
@@ -176,8 +172,8 @@ static int client_get_request (GtTransfer *xfer)
 	snprintf (host, sizeof (host) - 1, "%s:%hu", net_ip_str (xfer->ip),
 	          xfer->port);
 
-	/* always send the Range request just because we always know the full
-	 * size we want */
+	/* always send the Range request just because we always know the full size
+	 * we want */
 	ret = gt_http_client_send (c, "GET",        xfer->request,
 	                           "Range",         range_hdr,
 	                           "Host",          host,
@@ -189,21 +185,24 @@ static int client_get_request (GtTransfer *xfer)
 }
 
 /*
- * Verify connection status and Send the GET request to the server
+ * Verify connection status and Send the GET request to the server.
  */
-void gt_http_client_start (int fd, input_id id, TCPC *c)
+void gt_http_client_start (int fd, input_id id, GtTransfer *xfer)
 {
-	Chunk       *chunk = NULL;
-	GtTransfer  *xfer  = NULL;
-	GtSource    *src;
+	Chunk    *chunk;
+	TCPC     *c;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
-	src = gt_transfer_get_source (xfer);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	if (net_sock_error (c->fd))
 	{
+		GtSource *gt;
+
+		gt = gt_transfer_get_source (xfer);
+
 		/* set the connection as having failed, and retry w/ a push */
-		src->connect_failed = TRUE;
+		gt->connect_failed = TRUE;
 
 		gt_transfer_status (xfer, SOURCE_CANCELLED, (fd == -1 ?
 		                                             "Connection timed out" :
@@ -238,8 +237,8 @@ void gt_http_client_start (int fd, input_id id, TCPC *c)
 	input_remove (id);
 
 	/* wait for the server response */
-	input_add (fd, c, INPUT_READ,
-			   (InputCallback) get_server_reply, TIMEOUT_DEF);
+	input_add (fd, xfer, INPUT_READ,
+			   (InputCallback)get_server_reply, TIMEOUT_DEF);
 }
 
 /*
@@ -247,18 +246,17 @@ void gt_http_client_start (int fd, input_id id, TCPC *c)
  * Note that the GtTransfer can still timeout, in which case
  * the connection will get closed and so will the xfer.
  */
-static void read_response_body (int fd, input_id id, TCPC *c)
+static void read_response_body (int fd, input_id id, GtTransfer *xfer)
 {
-	GtTransfer  *xfer  = NULL;
-	Chunk       *chunk = NULL;
+	Chunk       *chunk;
+	TCPC        *c;
 	FDBuf       *buf;
 	char        *response;
 	int          n;
 	int          len;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
-	assert (xfer != NULL);
-	assert (chunk != NULL);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	len = xfer->stop - xfer->start;
 
@@ -460,9 +458,9 @@ static void set_queue_status (GtTransfer *xfer)
 static void handle_http_error (GtTransfer *xfer, SourceStatus status,
                                char *status_txt)
 {
-	TCPC    *c     = NULL;
-	Chunk   *chunk = NULL;
-	int      len   = 0;
+	TCPC    *c;
+	Chunk   *chunk;
+	int      len         = 0;
 	char    *content_len;
 	char    *conn_hdr;
 
@@ -472,9 +470,8 @@ static void handle_http_error (GtTransfer *xfer, SourceStatus status,
 
 	free (status_txt);
 
-	gt_transfer_unref (&c, &chunk, &xfer);
-	assert (chunk != NULL);
-	assert (c != NULL);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	/*
 	 * Check for a Content-Length: field, and use that for the
@@ -527,7 +524,7 @@ static void handle_http_error (GtTransfer *xfer, SourceStatus status,
 	}
 
 	input_remove_all (c->fd);
-	input_add (c->fd, c, INPUT_READ,
+	input_add (c->fd, xfer, INPUT_READ,
 	           (InputCallback)read_response_body, TIMEOUT_DEF);
 }
 
@@ -546,15 +543,16 @@ static void handle_http_error (GtTransfer *xfer, SourceStatus status,
  */
 int gt_http_handle_code (GtTransfer *xfer, int code)
 {
-	TCPC     *c     = NULL;
-	Chunk    *chunk = NULL;
+	TCPC     *c;
+	Chunk    *chunk;
 	GtSource *gt;
 
 	/* successful code, do nothing */
 	if (code >= 200 && code <= 299)
 		return TRUE;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	gt = gt_transfer_get_source (xfer);
 	assert (gt != NULL);
@@ -566,6 +564,7 @@ int gt_http_handle_code (GtTransfer *xfer, int code)
 	 */
 	switch (code)
 	{
+	 default:
 	 case 404:                     /* not found */
 		{
 			/*
@@ -605,12 +604,6 @@ int gt_http_handle_code (GtTransfer *xfer, int code)
 		 */
 		GT->source_abort (GT, chunk->transfer, xfer->source);
 		break;
-	 default:
-		GT->DBGFN (GT, "wtf? %i...", code);
-		gt_transfer_status (xfer, SOURCE_CANCELLED,
-		                    stringf ("Unknown error %i", code));
-		gt_transfer_close (xfer, TRUE);
-		break;
 	}
 
 	return FALSE;
@@ -740,10 +733,10 @@ static int verify_range_response (GtTransfer *xfer, Chunk *chunk)
 /*
  * Receive and process the HTTP response
  */
-static void get_server_reply (int fd, input_id id, TCPC *c)
+static void get_server_reply (int fd, input_id id, GtTransfer *xfer)
 {
-	Chunk         *chunk    = NULL;
-	GtTransfer    *xfer     = NULL;
+	Chunk         *chunk;
+	TCPC          *c;
 	GtSource      *gt;
 	FDBuf         *buf;
 	unsigned char *data;
@@ -751,9 +744,8 @@ static void get_server_reply (int fd, input_id id, TCPC *c)
 	size_t         data_len = 0;
 	int            n;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
-	assert (chunk != NULL);
-	assert (xfer != NULL);
+	chunk = gt_transfer_get_chunk (xfer);
+	c     = gt_transfer_get_tcpc  (xfer);
 
 	gt = gt_transfer_get_source (xfer);
 
@@ -780,12 +772,18 @@ static void get_server_reply (int fd, input_id id, TCPC *c)
 		return;
 	}
 
+	if (gt_fdbuf_full (buf))
+	{
+		gt_transfer_close (xfer, TRUE);
+		return;
+	}
+
 	if (n > 0)
 		return;
 
 	data = fdbuf_data (buf, &data_len);
 
-	if (!http_headers_terminated (data, data_len))
+	if (!gt_http_header_terminated (data, data_len))
 		return;
 
 	fdbuf_release (buf);
@@ -798,10 +796,6 @@ static void get_server_reply (int fd, input_id id, TCPC *c)
 		return;
 	}
 
-	/* check if the client is using a newer version than we are */
-	gt_version_warn_if_newer (xfer->ip,
-	                          dataset_lookupstr (xfer->header, "user-agent"));
-
 	/*
 	 * NOTE: if we wanted to do any further processing of the server reply
 	 * after GET /, this is where it would be
@@ -829,24 +823,28 @@ static void get_server_reply (int fd, input_id id, TCPC *c)
 		return;
 	}
 
+	/* disable the header read timeout */
+	timer_remove_zero (&xfer->header_timer);
+
 	/* wait for the file to be sent by the server */
 	input_remove (id);
-	input_add (fd, c, INPUT_READ,
-	           (InputCallback) gt_get_read_file, 0);
+	input_add (fd, xfer, INPUT_READ,
+	           (InputCallback)gt_get_read_file, 0);
 }
 
 /*
- * Receive the requested file from the server
+ * Receive the requested file from the server.
  */
-void gt_get_read_file (int fd, input_id id, TCPC *c)
+void gt_get_read_file (int fd, input_id id, GtTransfer *xfer)
 {
-	Chunk        *chunk = NULL;
-	GtTransfer   *xfer  = NULL;
+	TCPC         *c;
+	Chunk        *chunk;
 	char          buf[RW_BUFFER];
 	size_t        size;
 	int           recv_len;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	/* set the max initial size of the request */
 	size = sizeof (buf);
@@ -891,7 +889,7 @@ static void client_reset_timeout (int fd, input_id id, TCPC *c)
 	if (HTTP_DEBUG)
 		GT->DBGSOCK (GT, c, "closing client HTTP connection");
 
-	gt_http_connection_close (c, TRUE, GT_TRANSFER_DOWNLOAD);
+	gt_http_connection_close (GT_TRANSFER_DOWNLOAD, c, TRUE);
 }
 
 void gt_http_client_reset (TCPC *c)
@@ -905,5 +903,5 @@ void gt_http_client_reset (TCPC *c)
 
 	input_remove_all (c->fd);
 	input_add (c->fd, c, INPUT_READ,
-	           (InputCallback) client_reset_timeout, 2 * MINUTES);
+	           (InputCallback)client_reset_timeout, 2 * MINUTES);
 }
