@@ -1,5 +1,5 @@
 /*
- * $Id: gt_http_client.h,v 1.8 2003/11/27 23:18:03 hipnod Exp $
+ * $Id: gt_http_client.h,v 1.9 2004/01/18 05:40:56 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -19,14 +19,14 @@
 
 /*****************************************************************************/
 
-void gt_http_client_get   (Chunk *chunk, GtTransfer *xfer);
-void gt_http_client_push  (in_addr_t ip, in_port_t port,
-                           char *request, off_t start, off_t stop);
-void gt_http_client_reset (TCPC *c);
+void    gt_http_client_get     (Chunk *chunk, GtTransfer *xfer);
+void    gt_http_client_push    (in_addr_t ip, in_port_t port,
+                                char *request, off_t start, off_t stop);
+void    gt_http_client_reset   (TCPC *c);
 
-int gt_http_handle_code (GtTransfer *xfer, int code);
-void gt_http_client_start (int fd, input_id id, TCPC *c);
-void gt_get_read_file     (int fd, input_id id, TCPC *c);
+int     gt_http_handle_code    (GtTransfer *xfer, int code);
+void    gt_http_client_start   (int fd, input_id id, GtTransfer *xfer);
+void    gt_get_read_file       (int fd, input_id id, GtTransfer *xfer);
 
 /*****************************************************************************/
 
