@@ -1,5 +1,5 @@
 /*
- * $Id: gt_http_server.c,v 1.63 2003/12/22 02:50:42 hipnod Exp $
+ * $Id: gt_http_server.c,v 1.73 2005/01/04 15:03:40 mkern Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -28,7 +28,25 @@
 
 /*****************************************************************************/
 
+/* convenient shorthand */
+#define CONTENT_URN_FIELD       "X-Gnutella-Content-URN"
+
+#define INCOMING_TIMEOUT        (1 * MINUTES)
+
+/*****************************************************************************/
+
+struct http_incoming
+{
+	TCPC     *c;
+	timer_id  timer;
+};
+
+/*****************************************************************************/
+
 static void server_handle_get    (GtTransfer *xfer);
+static void get_client_request   (int fd, input_id id,
+                                  struct http_incoming *http);
+static void send_http_response   (int fd, input_id id, GtTransfer *xfer);
 
 /*****************************************************************************/
 /* SERVER HELPERS */
@@ -72,21 +90,10 @@ static char *lookup_http_code (int code, char **desc)
 	return err;
 }
 
-/*
- * Construct and send a server reply
- *
- * NOTE:
- * A similar function is available in http_client.c
- */
-static BOOL gt_http_server_send (TCPC *c, int code, ...)
+static String *alloc_header (int code)
 {
-	char       *key;
-	char       *value;
-	char       *code_text;
-	String     *s;
-	int         ret;
-	size_t      len;
-	va_list     args;
+	char   *code_text;
+	String *s;
 
 	/* so that we can communicate both the numerical code and the human
 	 * readable string */
@@ -98,11 +105,17 @@ static BOOL gt_http_server_send (TCPC *c, int code, ...)
 
 	string_appendf (s, "HTTP/1.1 %i %s\r\n", code, code_text);
 
+	return s;
+}
+
+static void construct_header_va (String *s, int code, va_list args)
+{
+	char *key;
+	char *value;
+
 	/* Add "Server: " header */
 	string_appendf (s, "Server: %s\r\n", gt_version ());
 
-	va_start (args, code);
-
 	for (;;)
 	{
 		if (!(key = va_arg (args, char *)))
@@ -114,10 +127,43 @@ static BOOL gt_http_server_send (TCPC *c, int code, ...)
 		string_appendf (s, "%s: %s\r\n", key, value);
 	}
 
-	va_end (args);
-
 	/* append final message terminator */
 	string_append (s, "\r\n");
+}
+
+static String *construct_header (int code, ...)
+{
+	String *s;
+	va_list args;
+
+	if (!(s = alloc_header (code)))
+		return NULL;
+
+	va_start (args, code);
+	construct_header_va (s, code, args);
+	va_end (args);
+
+	return s;
+}
+
+/*
+ * Construct and send a server reply.
+ */
+static BOOL gt_http_server_send (TCPC *c, int code, ...)
+{
+	String *s;
+	int     ret;
+	size_t  len;
+	va_list args;
+
+	if (!(s = alloc_header (code)))
+		return FALSE;
+
+	va_start (args, code);
+
+	construct_header_va (s, code, args);
+
+	va_end (args);
 
 	if (HTTP_DEBUG)
 		GT->DBGSOCK (GT, c, "sending reply to client =\n%s", s->str);
@@ -130,7 +176,7 @@ static BOOL gt_http_server_send (TCPC *c, int code, ...)
 	return (ret == len);
 }
 
-static char *get_error_page (TCPC *c, int code)
+static char *get_error_page (GtTransfer *xfer, int code)
 {
 	char *page;
 	char *err;
@@ -144,24 +190,21 @@ static char *get_error_page (TCPC *c, int code)
 	return page;
 }
 
-static void get_content_urns (GtTransfer *xfer, char **key, char **val)
-{
-	*key = NULL;
-	*val = NULL;
-
-	if (xfer && xfer->content_urns)
-	{
-		*key = "X-Gnutella-Content-URN";
-		*val = xfer->content_urns;
-	}
-}
-
 static BOOL supports_queue (GtTransfer *xfer)
 {
-	if (!(dataset_lookupstr (xfer->header, "x-queue")))
-		return FALSE;
+	char *features;
 
-	return TRUE;
+	if (dataset_lookupstr (xfer->header, "x-queue"))
+		return TRUE;
+
+	if ((features = dataset_lookupstr (xfer->header, "x-features")))
+	{
+		/* XXX: case-sensitive */
+		if (strstr (features, "queue"))
+			return TRUE;
+	}
+
+	return FALSE;
 }
 
 static char *get_queue_line (GtTransfer *xfer)
@@ -175,60 +218,110 @@ static char *get_queue_line (GtTransfer *xfer)
 	if (!(s = string_new (NULL, 0, 0, TRUE)))
 		return NULL;
 
-	string_appendf (s, "position=%d,length=%d,pollMin=%d,pollMax=%d", 
+	string_appendf (s, "position=%d,length=%d,pollMin=%d,pollMax=%d",
 	                xfer->queue_pos, xfer->queue_ttl, 45, 120);
 
 	return string_free_keep (s);
 }
 
-static int gt_http_server_send_error (TCPC *c, int code)
+static String *get_error_header (GtTransfer *xfer, int code,
+                                 const char *error_page)
 {
-	char        *error_page;
-	int          len;
-	int          sent_len;
-	char         content_len[256];
-	GtTransfer  *xfer            = NULL;
-	char        *queue_line      = NULL;
-	char        *content_urn_key = NULL;
-	char        *content_urn_val = NULL;
-
-	gt_transfer_unref (&c, NULL, &xfer);
-
-	error_page = get_error_page (c, code);
-	assert (error_page != NULL);
+	size_t  len;
+	char    content_len[256];
+	char   *queue_line      = NULL;
+	char   *content_type    = "text/html";
+	String *s;
 
 	len = strlen (error_page);
-	snprintf (content_len, sizeof (content_len) - 1, "%u", len);
-
-	/* Append any content urns */
-	get_content_urns (xfer, &content_urn_key, &content_urn_val);
+	snprintf (content_len, sizeof (content_len), "%u", len);
 
 	if (code == 503 && supports_queue (xfer))
 		queue_line = get_queue_line (xfer);
 
-	gt_http_server_send (c, code,
-	                     "Content-Type",    "text/html",
-	                     "Content-Length",   content_len,
-	                     content_urn_key,    content_urn_val,
-	                     "X-Queue",          queue_line,
-	                     NULL);
+	/* don't send a content-type header if there is no entity body */
+	if (len == 0)
+		content_type = NULL;
 
-	sent_len = tcp_send (c, error_page, len);
+	s = construct_header (code,
+	                      "Content-Type",    content_type,
+	                      "Content-Length",  content_len,
+	                      CONTENT_URN_FIELD, xfer->content_urns,
+	                      "X-Queue",         queue_line,
+	                      NULL);
 
-	/* if the whole page was sent, then its safe to use persistent
-	 * http by setting the headers to transmitted and the remaining len to
-	 * zero */
-	if (sent_len == len)
+	free (queue_line);
+	return s;
+}
+
+static void send_error_reply (int fd, input_id id, GtTransfer *xfer)
+{
+	String     *s;
+	const char *error_page;
+	int         ret;
+	TCPC       *c;
+
+	c = gt_transfer_get_tcpc (xfer);
+
+	if (!(error_page = get_error_page (xfer, xfer->code)))
+	{
+		gt_transfer_close (xfer, TRUE);
+		return;
+	}
+
+	/*
+	 * If the remote end supports queueing or supplied
+	 * "X-Gnutella-Content-URN",, it's a Gnutella client and we don't want to
+	 * keep sending the error page, because over many requests from this
+	 * client the bandwidth could add up.
+	 */
+	if (supports_queue (xfer) ||
+	    dataset_lookupstr (xfer->header, "x-gnutella-content-urn"))
+	{
+		error_page = ""; /* empty */
+	}
+
+	if (!(s = get_error_header (xfer, xfer->code, error_page)))
+	{
+		gt_transfer_close (xfer, TRUE);
+		return;
+	}
+
+	string_append (s, error_page);
+
+	if (HTTP_DEBUG)
+		GT->DBGSOCK (GT, c, "sending reply to client =\n%s", s->str);
+
+	/* send the whole page at once */
+	ret = tcp_send (c, s->str, s->len);
+
+	/* if the whole thing was sent keep the connection open */
+	if (ret == s->len)
 	{
 		xfer->transmitted_hdrs = TRUE;
 		xfer->remaining_len    = 0;
 	}
 
-	free (queue_line);
+	string_free (s);
 
-	return sent_len;
+	gt_transfer_close (xfer, FALSE);
 }
 
+static void gt_http_server_send_error_and_close (GtTransfer *xfer, int code)
+{
+	TCPC *c;
+
+	c = gt_transfer_get_tcpc (xfer);
+
+	xfer->code = code;
+
+	input_remove_all (c->fd);
+	input_add (c->fd, xfer, INPUT_WRITE,
+	           (InputCallback)send_error_reply, TIMEOUT_DEF);
+}
+
+/*****************************************************************************/
+
 /* parse the Range: bytes=0-10000 format */
 static void parse_client_request_range (Dataset *dataset,
                                         off_t *r_start, off_t *r_stop)
@@ -266,11 +359,10 @@ static void parse_client_request_range (Dataset *dataset,
 	stop  = (off_t) ATOI (string_sep (&range, " "));
 
 	/*
-	 * The end of the range is optional (e.g. "Range: 0-"),
-	 * and in that case stop == 0. In the case of a single
-	 * byte file, stop == 1.
+	 * The end of the range is optional (e.g. "Range: 0-"), and in that case
+	 * stop == 0. In the case of a single byte file, stop == 1.
 	 *
-	 * TODO: this is broken for one-byte requests.
+	 * TODO: this is broken for one-byte requests at the start of the file.
 	 */
 	if (stop > 0)
 		stop = stop + 1;
@@ -281,16 +373,16 @@ static void parse_client_request_range (Dataset *dataset,
 		*r_stop = stop;
 }
 
-/* 
- * Parse requests of the forms: 
- *  
+/*
+ * Parse requests of the forms:
+ *
  *  /get/47/File maybe with spaces HTTP
  *  /get/47/files_%20_url_encoded HTTP/1.1
  *  /uri-res/N2R?urn:sha1:ABCD HTTP/1.0
  *
  * and combinations thereof. The "HTTP" trailer is mandatory.
  */
-static void get_request_and_version (char *data, char **request, 
+static void get_request_and_version (char *data, char **request,
                                      char **version)
 {
 	size_t  len;
@@ -301,10 +393,12 @@ static void get_request_and_version (char *data, char **request,
 	*request = NULL;
 	*version = NULL;
 
+	/* trim whitespace inbetween command and request */
+	string_trim (data);
+
 	if (!(dup = STRDUP (data)))
 		return;
 
-	string_trim (dup);
 	string_upper (dup);
 
 	next = dup;
@@ -359,8 +453,11 @@ static int parse_client_request (Dataset **r_dataset, char **r_command,
 	 */
 	get_request_and_version (req_line, &request, &version);
 
-	GT->DBGFN (GT, "command=%s version=%s request=%s",
-	           command, version, request);
+	if (HTTP_DEBUG)
+	{
+		GT->DBGFN (GT, "command=%s version=%s request=%s",
+		           command, version, request);
+	}
 
 	if (!request || string_isempty (request))
 		return FALSE;
@@ -372,7 +469,7 @@ static int parse_client_request (Dataset **r_dataset, char **r_command,
 	if (r_version)
 		*r_version = version;
 
-	http_headers_parse (hdr, &dataset);
+	gt_http_header_parse (hdr, &dataset);
 
 	if (r_dataset)
 		*r_dataset = dataset;
@@ -399,19 +496,18 @@ static int parse_client_request (Dataset **r_dataset, char **r_command,
  */
 static void reply_to_client_request (GtTransfer *xfer)
 {
-	TCPC       *c     = NULL;
-	Chunk      *chunk = NULL;
+	TCPC       *c;
+	Chunk      *chunk;
 	off_t       entity_size;
 	char        range[128];
 	char        length[32];
-	char       *content_urn_key;
-	char       *content_urn_val;
 	BOOL        ret;
 
 	if (!xfer)
 		return;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
+	c     = gt_transfer_get_tcpc  (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	/*
 	 * Determine the "total" entity body that we have locally, not necessarily
@@ -435,13 +531,11 @@ static void reply_to_client_request (GtTransfer *xfer)
 	snprintf (length, sizeof (length) - 1, "%i",
 	          (int) (xfer->stop - xfer->start));
 
-	get_content_urns (xfer, &content_urn_key, &content_urn_val);
-
 	ret = gt_http_server_send (c, xfer->code,
-	                           "Content-Range",  range,
-	                           "Content-Length", length,
-	                           "Content-Type",   xfer->content_type,
-	                           content_urn_key,  content_urn_val,
+	                           "Content-Range",   range,
+	                           "Content-Length",  length,
+	                           "Content-Type",    xfer->content_type,
+	                           CONTENT_URN_FIELD, xfer->content_urns,
 	                           NULL);
 
 	/* if we transmitted all headers successfully, set transmitted_hdrs
@@ -452,12 +546,71 @@ static void reply_to_client_request (GtTransfer *xfer)
 
 /*****************************************************************************/
 
+static void http_incoming_free (struct http_incoming *incoming)
+{
+	timer_remove (incoming->timer);
+	free (incoming);
+}
+
+static void http_incoming_close (struct http_incoming *incoming)
+{
+	gt_http_connection_close (GT_TRANSFER_UPLOAD, incoming->c, TRUE);
+	http_incoming_free (incoming);
+}
+
+static BOOL http_incoming_timeout (struct http_incoming *incoming)
+{
+	http_incoming_close (incoming);
+	return FALSE;
+}
+
+static struct http_incoming *http_incoming_alloc (TCPC *c)
+{
+	struct http_incoming *incoming;
+
+	incoming = malloc (sizeof (struct http_incoming));
+	if (!incoming)
+		return NULL;
+
+	incoming->c = c;
+	incoming->timer = timer_add (INCOMING_TIMEOUT,
+	                             (TimerCallback)http_incoming_timeout,
+	                             incoming);
+
+	return incoming;
+}
+
+void gt_http_server_dispatch (int fd, input_id id, TCPC *c)
+{
+	struct http_incoming *incoming;
+
+	if (net_sock_error (c->fd))
+	{
+		gt_http_connection_close (GT_TRANSFER_UPLOAD, c, TRUE);
+		return;
+	}
+
+	if (!(incoming = http_incoming_alloc (c)))
+	{
+		gt_http_connection_close (GT_TRANSFER_UPLOAD, c, TRUE);
+		return;
+	}
+
+	/* keep track of this incoming connection */
+	/* gt_http_connection_insert (GT_TRANSFER_UPLOAD, c); */
+
+	input_remove (id);
+	input_add (c->fd, incoming, INPUT_READ,
+	           (InputCallback)get_client_request, 0);
+}
+
 /*
- * Handle the client's GET or PUSH commands
+ * Handle the client's GET commands.
  */
-void gt_get_client_request (int fd, input_id id, TCPC *c)
+static void get_client_request (int fd, input_id id, struct http_incoming *http)
 {
 	GtTransfer    *xfer;
+	TCPC          *c;
 	Dataset       *dataset = NULL;
 	char          *command = NULL;
 	char          *request = NULL;
@@ -469,11 +622,18 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 	size_t         data_len = 0;
 	int            n;
 
+	c = http->c;
 	buf = tcp_readbuf (c);
 
 	if ((n = fdbuf_delim (buf, "\n")) < 0)
 	{
-		gt_http_connection_close (c, TRUE, GT_TRANSFER_UPLOAD);
+		http_incoming_close (http);
+		return;
+	}
+
+	if (gt_fdbuf_full (buf))
+	{
+		http_incoming_close (http);
 		return;
 	}
 
@@ -482,8 +642,7 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 
 	data = fdbuf_data (buf, &data_len);
 
-	/* TODO: this should be timed out */
-	if (!http_headers_terminated (data, data_len))
+	if (!gt_http_header_terminated (data, data_len))
 		return;
 
 	fdbuf_release (buf);
@@ -491,15 +650,18 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 	if (HTTP_DEBUG)
 		GT->DBGSOCK (GT, c, "client request:\n%s", data);
 
-	/* parse the client's reply and determine how we should proceed */
+	/* parse the client's request and determine how we should proceed */
 	if (!parse_client_request (&dataset, &command, &request, &version,
 	                           &start, &stop, data))
 	{
 		GT->DBGSOCK (GT, c, "invalid http header");
-		gt_http_connection_close (c, TRUE, GT_TRANSFER_UPLOAD);
+		http_incoming_close (http);
 		return;
 	}
 
+	/* discard incoming connection timeout maintainance structure */
+	http_incoming_free (http);
+
 	/*
 	 * We have enough information now to actually allocate the transfer
 	 * structure and pass it along to all logic that follows this
@@ -511,7 +673,8 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 	xfer = gt_transfer_new (GT_TRANSFER_UPLOAD, NULL,
 	                        net_peer (c->fd), 0, start, stop);
 
-	gt_transfer_ref (c, NULL, xfer);
+	/* connect the connection and the xfer in unholy matrimony */
+	gt_transfer_set_tcpc (xfer, c);
 
 	/* assign all our own memory */
 	xfer->command = STRDUP (command);
@@ -521,16 +684,12 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 	if (!gt_transfer_set_request (xfer, request))
 	{
 		if (HTTP_DEBUG)
-			GT->DBGSOCK (GT, c, "invalid request '%s'", request);
+			GT->DBGSOCK (GT, c, "invalid request \"s\"", request);
 
 		gt_transfer_close (xfer, TRUE);
 		return;
 	}
 
-	/* check if the client is using a newer version than we are */
-	gt_version_warn_if_newer (xfer->ip,
-	                          dataset_lookupstr (xfer->header, "User-Agent"));
-
 	/* no need for this function again */
 	input_remove (id);
 
@@ -542,10 +701,11 @@ void gt_get_client_request (int fd, input_id id, TCPC *c)
 		return;
 	}
 
-	gt_http_server_send_error (c, 501);
-	gt_transfer_close (xfer, FALSE);
+	gt_http_server_send_error_and_close (xfer, 501);
 }
 
+/*****************************************************************************/
+
 static Transfer *start_upload (GtTransfer *xfer, Chunk **chunk)
 {
 	Transfer *transfer;
@@ -567,12 +727,13 @@ int gt_server_setup_upload (GtTransfer *xfer)
 {
 	Transfer   *transfer;              /* giFT structure */
 	Chunk      *chunk;
-	TCPC       *c = NULL;
+	TCPC       *c;
 
 	if (!xfer)
 		return FALSE;
 
-	gt_transfer_unref (&c, NULL, &xfer);
+	c = gt_transfer_get_tcpc (xfer);
+	assert (xfer->chunk == NULL);
 
 	/*
 	 * Ban the host if they don't have access -- this gives no information
@@ -620,8 +781,8 @@ int gt_server_setup_upload (GtTransfer *xfer)
 	if (xfer->remaining_len != xfer->share_authd->size)
 		xfer->code = 206;
 
-	/* assign the circular references for passing this data along */
-	gt_transfer_ref (c, chunk, xfer);
+	/* assign the circular references for passing the chunk along */
+	gt_transfer_set_chunk (xfer, chunk);
 
 	/* finally, seek the file descriptor where it needs to be */
 	fseek (xfer->f, xfer->start, SEEK_SET);
@@ -629,41 +790,40 @@ int gt_server_setup_upload (GtTransfer *xfer)
 	return TRUE;
 }
 
-/*
- * Serve client GET / requests
- */
 static void server_handle_get (GtTransfer *xfer)
 {
-	TCPC *c = NULL;
+	TCPC *c;
 
-	if (!xfer)
-		return;
-
-	gt_transfer_unref (&c, NULL, &xfer);
+	c = gt_transfer_get_tcpc (xfer);
+	assert (xfer->chunk == NULL);
 
 	/* WARNING: this block is duplicated in http_client:client_push_request */
 	if (!gt_server_setup_upload (xfer))
 	{
-		/* gracefully reject */
-		switch (xfer->code)
-		{
-		 case 503:
-			gt_http_server_send_error (c, xfer->code);
-			break;
-		 case 200:                     /* general failure */
+		if (xfer->code == 200)
 			xfer->code = 404;
-			/* fall through */
-		 default:
-			gt_http_server_send_error (c, xfer->code);
-			break;
-		}
 
-		/* perform an orderly close: keep the connection alive if possible */
-		gt_transfer_close (xfer, FALSE);
+		gt_http_server_send_error_and_close (xfer, xfer->code);
 		return;
 	}
 
-	/* ok, give the client the accepted header */
+	input_add (c->fd, xfer, INPUT_WRITE,
+	           (InputCallback)send_http_response, TIMEOUT_DEF);
+}
+
+static void send_http_response (int fd, input_id id, GtTransfer *xfer)
+{
+	TCPC *c;
+
+	c = gt_transfer_get_tcpc (xfer);
+
+	if (net_sock_error (c->fd))
+	{
+		gt_transfer_close (xfer, TRUE);
+		return;
+	}
+
+	/* ok, send client the header */
 	reply_to_client_request (xfer);
 
 	if (!strcasecmp (xfer->command, "HEAD"))
@@ -672,24 +832,29 @@ static void server_handle_get (GtTransfer *xfer)
 		return;
 	}
 
-	input_add (c->fd, c, INPUT_WRITE,
-			   (InputCallback) gt_server_upload_file, 0);
+	/* disable header read timer */
+	timer_remove_zero (&xfer->header_timer);
+
+	input_remove (id);
+	input_add (c->fd, xfer, INPUT_WRITE,
+			   (InputCallback)gt_server_upload_file, 0);
 }
 
 /*
  * Uploads the file requests
  */
-void gt_server_upload_file (int fd, input_id id, TCPC *c)
+void gt_server_upload_file (int fd, input_id id, GtTransfer *xfer)
 {
-	Chunk       *chunk = NULL;
-	GtTransfer  *xfer  = NULL;
+	TCPC        *c;
+	Chunk       *chunk;
 	char         buf[RW_BUFFER];
 	size_t       read_len;
 	size_t       size;
 	int          sent_len = 0;
 	off_t        remainder;
 
-	gt_transfer_unref (&c, &chunk, &xfer);
+	c     = gt_transfer_get_tcpc (xfer);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	assert (xfer->f != NULL);
 
@@ -742,7 +907,7 @@ void gt_server_upload_file (int fd, input_id id, TCPC *c)
 		return;
 	}
 
-	/* 
+	/*
 	 * Check for short send(). This could use fseek(), but I have this
 	 * growing feeling that using stdio everywhere is a bad idea.
 	 */
@@ -753,20 +918,21 @@ void gt_server_upload_file (int fd, input_id id, TCPC *c)
 		return;
 	}
 
-	/* 
+	/*
 	 * Call gt_upload to report back to giFT. This will also cancel
 	 * the transfer if the upload has completed.
 	 */
 	gt_transfer_write (xfer, chunk, buf, sent_len);
-
 }
 
 /*****************************************************************************/
 
 void gt_http_server_reset (TCPC *c)
 {
-	/* This can happen because the GtTransfer and TCPC can be decoupled.
-	 * as in the case of a push request sent. */
+	/*
+	 * This can happen because the GtTransfer and TCPC can be decoupled, as in
+	 * the case of a push request sent.
+	 */
 	if (!c)
 		return;
 
@@ -776,5 +942,5 @@ void gt_http_server_reset (TCPC *c)
 	/* reset the input state */
 	input_remove_all (c->fd);
 	input_add (c->fd, c, INPUT_READ,
-			   (InputCallback) gt_get_client_request, 2 * MINUTES);
+			   (InputCallback)gt_http_server_dispatch, 2 * MINUTES);
 }
