@@ -1,5 +1,5 @@
 /*
- * $Id: gt_http_server.h,v 1.5 2003/11/27 23:18:03 hipnod Exp $
+ * $Id: gt_http_server.h,v 1.7 2004/02/23 04:20:07 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -19,14 +19,13 @@
 
 /*****************************************************************************/
 
-void gt_http_server_incoming (int fd, input_id id, TCPC *c);
+void    gt_http_server_incoming  (int fd, input_id id, TCPC *c);
 
-int  gt_server_setup_upload  (GtTransfer *xfer);
-void gt_server_upload_file   (int fd, input_id id, TCPC *c);
+int     gt_server_setup_upload   (GtTransfer *xfer);
+void    gt_server_upload_file    (int fd, input_id id, GtTransfer *xfer);
 
-void gt_http_server_reset    (TCPC *c);
-
-void gt_get_client_request (int fd, input_id id, TCPC *c);
+void    gt_http_server_dispatch  (int fd, input_id id, TCPC *c);
+void    gt_http_server_reset     (TCPC *c);
 
 /*****************************************************************************/
 
