@@ -1,5 +1,5 @@
 /*
- * $Id: gt_packet.c,v 1.26 2003/12/22 01:55:01 hipnod Exp $
+ * $Id: gt_packet.c,v 1.35 2004/04/17 06:05:53 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -17,7 +17,6 @@
 #include "gt_gnutella.h"
 
 #include "gt_packet.h"
-#include "gt_protocol.h"
 
 #include "gt_node.h"
 #include "gt_netorg.h"
@@ -25,6 +24,7 @@
 #include "gt_utils.h"
 
 #include "gt_query_route.h"
+#include "message/gt_message.h"  /* gt_vendor_msg_get_{id,version} */
 
 #include <stdarg.h>
 
@@ -82,6 +82,26 @@ GtPacket *gt_packet_reply (GtPacket *src, uint8_t cmd)
 	return packet;
 }
 
+GtPacket *gt_packet_vendor (const gt_vendor_msg_t *vmsg)
+{
+	GtPacket            *pkt;
+
+	if (!(pkt = gt_packet_new (GT_MSG_VENDOR, 1, NULL)))
+		return NULL;
+
+	gt_packet_put_ustr   (pkt, vmsg->vendor_id, 4);
+	gt_packet_put_uint16 (pkt, vmsg->id);
+	gt_packet_put_uint16 (pkt, 1); /* vmsg version */
+
+	if (gt_packet_error (pkt))
+	{
+		gt_packet_free (pkt);
+		return NULL;
+	}
+
+	return pkt;
+}
+
 void gt_packet_free (GtPacket *packet)
 {
 	if (!packet)
@@ -208,7 +228,7 @@ static uint32_t gt_get32 (unsigned char *src, int endian, int swap)
 
 /*****************************************************************************/
 
-static int gt_packet_resize (GtPacket *packet, size_t len)
+static BOOL gt_packet_resize (GtPacket *packet, size_t len)
 {
 	size_t         resize_len;
 	unsigned char *resized;
@@ -269,7 +289,7 @@ int gt_packet_seek (GtPacket *packet, int offset)
 	return old_offset;
 }
 
-GtPacket *gt_packet_unserialize (unsigned char *data, int len)
+GtPacket *gt_packet_unserialize (unsigned char *data, size_t len)
 {
 	GtPacket *packet;
 
@@ -363,7 +383,7 @@ uint32_t gt_packet_get_uint (GtPacket *packet, size_t size,
 
 /*****************************************************************************/
 
-static int gt_packet_append (GtPacket *packet, void *data, size_t size)
+static int gt_packet_append (GtPacket *packet, const void *data, size_t size)
 {
 	size_t required;
 
@@ -439,19 +459,22 @@ int gt_packet_put_ip (GtPacket *packet, in_addr_t ip)
 int gt_packet_put_port (GtPacket *packet, in_port_t port)
 { return gt_packet_put_uint (packet, &port, 2, LITTLE, TRUE); }
 
-int gt_packet_put_ustr (GtPacket *packet, unsigned char *str, int len)
+int gt_packet_put_ustr (GtPacket *packet, const unsigned char *str, size_t len)
 {
-	if (!str || len == 0)
-		return gt_packet_put_uint8 (packet, 0);
-
-	if (len == -1)
-		len = strlen (str) + 1;
-
-	return gt_packet_append (packet, str, (size_t)len);
+	assert (len > 0);
+	return gt_packet_append (packet, str, len);
 }
 
-int gt_packet_put_str (GtPacket *packet, char *str)
-{ return gt_packet_put_ustr (packet, (unsigned char *)str, -1); }
+int gt_packet_put_str (GtPacket *packet, const char *str)
+{
+	size_t len;
+
+	if (!str)
+		return gt_packet_put_uint8 (packet, 0);
+
+	len = strlen (str) + 1;
+	return gt_packet_put_ustr (packet, (const unsigned char *)str, len);
+}
 
 /*****************************************************************************/
 
@@ -567,7 +590,7 @@ char *gt_packet_get_str (GtPacket *packet)
 	return gt_packet_get_array (packet, 0, 1, TRUE, FALSE, FALSE);
 }
 
-unsigned char *gt_packet_get_ustr (GtPacket *packet, int len)
+unsigned char *gt_packet_get_ustr (GtPacket *packet, size_t len)
 {
 	return gt_packet_get_array (packet, len, 1, FALSE, FALSE, FALSE);
 }
@@ -580,13 +603,15 @@ static char *packet_command_str (unsigned char cmd)
 
 	switch (cmd)
 	{
-	 case GT_PING_REQUEST:    return "PING";
-	 case GT_PING_RESPONSE:   return "PONG";
-	 case GT_BYE_REQUEST:     return "BYE";
-	 case GT_QUERY_ROUTE:     return "QROUTE";
-	 case GT_PUSH_REQUEST:    return "PUSH";
-	 case GT_QUERY_REQUEST:   return "QUERY";
-	 case GT_QUERY_RESPONSE:  return "HITS";
+	 case GT_MSG_PING:        return "PING";
+	 case GT_MSG_PING_REPLY:  return "PONG";
+	 case GT_MSG_BYE:         return "BYE";
+	 case GT_MSG_QUERY_ROUTE: return "QROUTE";
+	 case GT_MSG_VENDOR:      return "VMSG";
+	 case GT_MSG_VENDOR_STD:  return "VMSG-S";
+	 case GT_MSG_PUSH:        return "PUSH";
+	 case GT_MSG_QUERY:       return "QUERY";
+	 case GT_MSG_QUERY_REPLY: return "HITS";
 
 	 default:
 		snprintf (buf, sizeof (buf), "[<%02hx>]", cmd);
@@ -658,7 +683,9 @@ int gt_packet_send (TCPC *c, GtPacket *packet)
 	if (!c || c->fd < 0)
 		return -1;
 
-	gt_node_queue (c, packet);
+	if (!gt_node_send (GT_NODE(c), packet))
+		return -1;
+
 	return 0;
 }
 
