@@ -1,5 +1,5 @@
 /*
- * $Id: gt_query_route.c,v 1.41 2003/12/22 02:48:16 hipnod Exp $
+ * $Id: gt_query_route.c,v 1.46 2004/04/05 07:56:54 hipnod Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -17,15 +17,12 @@
 #include "gt_gnutella.h"
 
 #include "gt_query_route.h"
-
 #include "gt_packet.h"
-#include "gt_protocol.h"
+#include "gt_utils.h"
 
 #include "gt_node.h"
 #include "gt_node_list.h"
 
-#include "gt_utils.h"
-
 #include <libgift/stopwatch.h>
 
 #include <zlib.h>
@@ -38,11 +35,11 @@
  *      - support incremental updates of table
  *      - cut-off entries when MAX_FILL_RATIO is reached
  */
-#define MIN_TABLE_BITS        (19)                          /* 19   bits  */
+#define MIN_TABLE_BITS        (16)                          /* 16   bits  */
 #define MAX_TABLE_BITS        (21)                          /* 21   bits  */
-#define MIN_TABLE_SIZE        (1UL << (MIN_TABLE_BITS - 1)) /* 256k bytes */
+#define MIN_TABLE_SIZE        (1UL << (MIN_TABLE_BITS - 1)) /* 32k  bytes */
 #define MAX_TABLE_SIZE        (1UL << (MAX_TABLE_BITS - 1)) /* 1M   bytes */
-#define MIN_TABLE_SLOTS       (MIN_TABLE_SIZE * 2)          /* 512k slots */
+#define MIN_TABLE_SLOTS       (MIN_TABLE_SIZE * 2)          /* 64k  slots */
 #define MAX_TABLE_SLOTS       (MAX_TABLE_SIZE * 2)          /* 2M   slots */
 #define INC_FILL_RATIO        (0.70)                        /* 0.7%       */
 #define MAX_FILL_RATIO        (1.00)                        /* 1% (unused)*/
@@ -63,7 +60,7 @@
 /*
  * How often we check to build a compressed patch of local shares.
  */
-#define QRT_BUILD_INTERVAL    (10 * SECONDS)
+#define QRT_BUILD_INTERVAL    (3 * SECONDS)
 
 /*
  * How long to wait after the first query_route_table_submit() before
@@ -351,9 +348,13 @@ static uint8_t *compress_table (uint8_t *table, size_t in_size, size_t *out_size
 
 	*out_size = 0;
 
-	if (!(out = MALLOC (sizeof (*out))))
+	if (!(out = MALLOC (sizeof(*out))))
 		return NULL;
 
+	out->zalloc = Z_NULL;
+	out->zfree  = Z_NULL;
+	out->opaque = Z_NULL;
+
 	if ((ret = deflateInit (out, Z_DEFAULT_COMPRESSION)) != Z_OK)
 	{
 		GT->DBGFN (GT, "deflateInit error: %s", zlib_strerror (ret));
@@ -1055,7 +1056,7 @@ static void submit_empty_table (TCPC *c)
 #endif
 
 	/* format: <query-route-msg-type> <length of table> <infinity> */
-	if (gt_packet_send_fmt (c, GT_QUERY_ROUTE, NULL, 1, 0,
+	if (gt_packet_send_fmt (c, GT_MSG_QUERY_ROUTE, NULL, 1, 0,
 	                        "%c%lu%c", 0, (unsigned long) sizeof (table),
 	                        INFINITY) < 0)
 	{
@@ -1065,7 +1066,7 @@ static void submit_empty_table (TCPC *c)
 
 	len = sizeof (table);
 
-	if (gt_packet_send_fmt (c, GT_QUERY_ROUTE, NULL, 1, 0,
+	if (gt_packet_send_fmt (c, GT_MSG_QUERY_ROUTE, NULL, 1, 0,
 	                        "%c%c%c%c%c%*p",
 	                        1, 1, 1, 0, 8, len, table) < 0)
 	{
@@ -1086,7 +1087,7 @@ static void submit_table (TCPC *c, uint8_t *table, size_t size, size_t slots)
 	/* XXX make table size settable at runtime */
 
 	/* send a reset table first */
-	if (gt_packet_send_fmt (c, GT_QUERY_ROUTE, NULL, 1, 0,
+	if (gt_packet_send_fmt (c, GT_MSG_QUERY_ROUTE, NULL, 1, 0,
 	                        "%c%lu%c", 0, (long)slots, infinity) < 0)
 	{
 		GT->DBGFN (GT, "error reseting table");
@@ -1111,7 +1112,7 @@ static void submit_table (TCPC *c, uint8_t *table, size_t size, size_t slots)
 	{
 		send_size = MIN (PATCH_FRAGSIZE, size);
 
-		if (gt_packet_send_fmt (c, GT_QUERY_ROUTE, NULL, 1, 0,
+		if (gt_packet_send_fmt (c, GT_MSG_QUERY_ROUTE, NULL, 1, 0,
 		                        "%c%c%c%c%c%*p",
 		                        /* QRP PATCH */ 1,
 		                        seq_num, seq_size, compressed,
