@@ -1,5 +1,5 @@
 /*
- * $Id: gt_web_cache.c,v 1.56 2003/12/31 16:57:46 hipnod Exp $
+ * $Id: gt_web_cache.c,v 1.64 2005/07/11 13:49:10 mkern Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -65,33 +65,6 @@ static void parse_urlfile_response  (HttpRequest *http_req, char *url_file);
 
 /*****************************************************************************/
 
-static BOOL parse_web_cache_url (char *value, char **r_host_name,
-                                 char **r_remote_path)
-{
-	char  *host_name;
-
-	string_sep (&value, "http://");
-
-	/* divide the url in two parts */
-	host_name = string_sep (&value, "/");
-
-	if (r_host_name)
-		*r_host_name = host_name;
-
-	if (r_remote_path)
-	{
-		if (value && *value)
-			*r_remote_path = value;
-		else
-			*r_remote_path = "";
-	}
-
-	if (!host_name)
-		return FALSE;
-
-	return TRUE;
-}
-
 /* parse the extended data in the webcaches file, now its just mtime */
 static BOOL parse_web_cache_value (char *value, time_t *r_atime)
 {
@@ -108,19 +81,41 @@ static BOOL parse_web_cache_value (char *value, time_t *r_atime)
 
 /*****************************************************************************/
 
-static void ban_webcache (HttpRequest *req)
+static char *new_webcache_url (const char *host, const char *path)
+{
+	return stringf_dup ("http://%s/%s", host, STRING_NOTNULL(path));
+}
+
+static void ban_webcache (HttpRequest *req, const char *why)
 {
 	char *url;
 
-	url = stringf_dup ("http://%s/%s", req->host, req->path);
+	url = new_webcache_url (req->host, req->path);
 	GT->dbg (GT, "banning webcache %s", url);
 
-	file_cache_insert (bad_caches, url, "");
+	file_cache_insert (bad_caches, url, why);
 	file_cache_sync (bad_caches);
 
 	free (url);
 }
 
+static void insert_webcache (const char *host_name, const char *remote_path,
+                             time_t atime)
+{
+	char *url;
+	char *field;
+
+	url   = new_webcache_url (host_name, remote_path);
+	field = stringf_dup ("%lu", atime);
+
+	file_cache_insert (web_caches, url, field);
+
+	free (url);
+	free (field);
+}
+
+/*****************************************************************************/
+
 static void handle_close_request (HttpRequest *req, int error_code)
 {
 	String *s;
@@ -133,25 +128,19 @@ static void handle_close_request (HttpRequest *req, int error_code)
 			GT->DBGFN (GT, "connect to server %s failed for some reason",
 			           req->host);
 		}
-		/* webcaches that issue 300 responses should get removed too */
-#if 0
-		else if (error_code >= 300 && error_code < 400)
-		{
-			/* redirect: need to handle this at a higher level, actually
-			 * this should also remove the cache */
-			GT->DBGFN (GT, "%d (redirect?) request: bummer, cant handle yet",
-			           error_code);
-		}
-#endif
 		else
 		{
+			char err[32];
+
+			snprintf (err, sizeof(err), "Received error %d", error_code);
+
 			/*
-			 * Not found, internal server error, or redirect: ban the server's
-			 * URL
+			 * Not found, internal server error, or too many redirects: ban
+			 * the server's URL
 			 */
-			GT->DBGFN (GT, "server %s returned error %i ", req->host,
+			GT->DBGFN (GT, "server %s returned error %i", req->host,
 			           error_code);
-			ban_webcache (req);
+			ban_webcache (req, err);
 		}
 	}
 
@@ -187,7 +176,7 @@ static void parse_hostfile_response (HttpRequest *http_req, char *host_file)
 	if (!strncasecmp (host_file, "ERROR", sizeof ("ERROR") - 1) ||
 	    host_file[0] == '<')
 	{
-		ban_webcache (http_req);
+		ban_webcache (http_req, "Malformed response content");
 		return;
 	}
 
@@ -202,10 +191,10 @@ static void parse_hostfile_response (HttpRequest *http_req, char *host_file)
 		ip   = net_ip (string_sep (&host, ":"));
 		port = ATOI   (host);
 
-		if (!port || !ip || ip == (in_addr_t) -1)
+		if (!port || !ip || ip == INADDR_NONE)
 			continue;
 
-		GT->DBGFN (GT, "registering %s:%hu (from cache %s)", net_ip_str (ip), 
+		GT->DBGFN (GT, "registering %s:%hu (from cache %s)", net_ip_str (ip),
 		           port, http_req->host);
 
 		/* register the hosts as ultrapeers */
@@ -234,9 +223,9 @@ static void parse_hostfile_response (HttpRequest *http_req, char *host_file)
 	/* save the nodes we added to disk so we dont hit the caches again */
 	gt_node_list_save ();
 
-	/* 
+	/*
 	 * Do an exponential backoff from the caches. If we were online and
-	 * able to receive data, we should be getting node information 
+	 * able to receive data, we should be getting node information
 	 * some other way now.
 	 */
 	if (hosts >= 5)
@@ -276,16 +265,13 @@ static void parse_urlfile_response (HttpRequest *http_req, char *url_file)
 		if (!host_name)
 			continue;
 
-		if (remote_path)
-			url = stringf ("http://%s/%s", host_name, remote_path);
-		else
-			url = stringf ("http://%s/", host_name);
+		url = stringf ("http://%s/%s", host_name, STRING_NOTNULL(remote_path));
 
 		/* if the webcache is already in our db, skip it */
 		if (file_cache_lookup (web_caches, url))
 			continue;
 
-		/* 
+		/*
 		 * Only allow caches to register two more caches: this
 		 * small number helps to avoid our list of caches getting
 		 * polluted.
@@ -313,7 +299,115 @@ static void end_request (HttpRequest *req, char *data)
 		abort ();
 }
 
-static BOOL handle_recv (HttpRequest *req, char *data, int len)
+/*****************************************************************************/
+
+/*
+ * Return TRUE if newname is in the same domain as oldname.  For example,
+ * "new.gwc.example.com", "example.com", and "cache.example.com" are all
+ * considered in the same domain as "www.example.com".
+ *
+ * This is called on redirects, to make sure the cache can't redirect to an
+ * innocent site as part of a DDoS attack.
+ */
+static BOOL in_same_domain (const char *oldname, const char *newname)
+{
+	return FALSE;
+#if 0
+	const char *p;
+	const char *largest = NULL;
+	int         periods = 0;
+
+	p = newname;
+
+	/* get the largest common substring */
+	while (p != NULL)
+	{
+		if ((largest = strstr (oldname, p)))
+			break;
+
+		/* advance to next domain part */
+		p = strchr (p + 1, '.');
+	}
+
+	if (!largest)
+		return FALSE;
+
+	/*
+	 * Make sure the substring matches completely to the end.  This will
+	 * actually fail when it shouldn't if one name includes the '.' toplevel
+	 * domain and one doesn't.  Oh well.
+	 */
+	if (strcmp (largest, p) != 0)
+		return FALSE;
+
+	/*
+	 * Count the number of periods to find the number of subdomains in the
+	 * largest common substring.
+	 */
+	for (p = largest; *p != 0; p++)
+	{
+		if (*p == '.')
+			periods++;
+	}
+
+	/*
+	 * If the last character is the root '.', subtract one, since we are
+	 * looking for the number of common subdomains, and the root is shared by
+	 * all names.
+	 */
+	if (largest[strlen (largest) - 1] == '.')
+		periods--;
+
+	/*
+	 * If there are two periods, at least two toplevel domains match.
+	 */
+	if (periods >= 2)
+		return TRUE;
+
+	/*
+	 * If there is only one period shared, the names MAY be in the same
+	 * domain: one of the names has to be completely contained within the
+	 * other, such as the case of "foo.example.com" and "example.com".
+	 */
+	if (periods == 1 &&
+	    (strcmp (largest, oldname) == 0 || strcmp (largest, newname) == 0))
+	{
+		return TRUE;
+	}
+
+	/* not in same domain */
+	return FALSE;
+#endif
+}
+
+/*
+ * Called to when the webcache sends a 300-level response with a provided
+ * Location: header.  Have to make sure the domain the cache directs us
+ * to is the same.
+ */
+static BOOL handle_redirect (HttpRequest *req, const char *new_host,
+                             const char *new_path)
+{
+	assert (new_host != NULL);
+
+	if (in_same_domain (req->host, new_host) == FALSE)
+		return FALSE;
+
+	/* might want to do something else if the ban list later becomes per host
+	 * rather than per URL */
+	ban_webcache (req, "Redirected");
+
+	GT->DBGFN (GT, "Redirecting to new webcache %s/%s", new_host, new_path);
+
+	insert_webcache (new_host, new_path, time (NULL));
+	file_cache_sync (web_caches);
+
+	return TRUE;
+}
+
+/*****************************************************************************/
+
+static BOOL handle_recv (HttpRequest *req, char *data, size_t len)
 {
 	String *s;
 
@@ -355,9 +449,6 @@ static BOOL handle_recv (HttpRequest *req, char *data, int len)
 
 static BOOL handle_add_headers (HttpRequest *req, Dataset **headers)
 {
-	/* without this, HTTP/1.1 uses persistent http */
-	dataset_insertstr (headers, "Connection", "close");
-
 	/* don't let intermediaries cache our request, I think */
 	dataset_insertstr (headers, "Cache-Control", "no-cache");
 
@@ -402,7 +493,7 @@ static BOOL parse_host_and_port (char **r_host, in_port_t *r_port)
 	return TRUE;
 }
 
-static TCPC *open_http_connection (const char *http_name)
+static TCPC *open_http_connection (HttpRequest *req, const char *http_name)
 {
 	in_addr_t       ip;
 	in_port_t       port;
@@ -434,11 +525,18 @@ static TCPC *open_http_connection (const char *http_name)
 
 	/* ip is in network-order already */
 	memcpy (&ip, host->h_addr, MIN (host->h_length, sizeof (ip)));
-	c = tcp_open (ip, port, FALSE);
 
+	if (net_match_host (ip, "LOCAL"))
+	{
+		free (str);
+		ban_webcache (req, "Resolved to local IP");
+		return NULL;
+	}
+
+	c = tcp_open (ip, port, FALSE);
 	if (!c)
 	{
-		GT->DBGFN (GT, "couldn't open connection to %s [%s]: %s", 
+		GT->DBGFN (GT, "couldn't open connection to %s [%s]: %s",
 		           http_name, net_ip_str (ip), GIFT_NETERROR());
 	}
 
@@ -488,7 +586,7 @@ static void check_dns_error (const char *name, HttpRequest *req)
 	if (error == HOST_NOT_FOUND && gt_proxy_server == NULL)
 	{
 		GT->DBGFN (GT, "webcache \"%s\" not in DNS. banning", name);
-		ban_webcache (req);
+		ban_webcache (req, "Host not found in DNS");
 		return;
 	}
 }
@@ -498,34 +596,44 @@ static BOOL make_request (char *host_name, char *remote_path, char *request)
 	HttpRequest    *req;
 	TCPC           *c;
 	char           *resolve_name;
+	char           *url;
 
-	if (!(req = http_request_new (host_name, remote_path, request)))
+	url = stringf_dup ("http://%s/%s", host_name, STRING_NOTNULL(remote_path));
+
+	if (!(req = gt_http_request_new (url, request)))
+	{
+		free (url);
 		return FALSE;
+	}
+
+	free (url);
 
 	resolve_name = get_http_name (host_name);
 
 	gt_dns_set_errno (0);
 
-	if (!(c = open_http_connection (resolve_name)))
+	if (!(c = open_http_connection (req, resolve_name)))
 	{
 		check_dns_error (resolve_name, req);
-		http_request_close (req, -1);
+		gt_http_request_close (req, -1);
 		return FALSE;
 	}
 
-	GT->DBGFN (GT, "opening connection to %s [%s]", 
+	GT->DBGFN (GT, "opening connection to %s [%s]",
 	           resolve_name, net_ip_str (c->host));
 
-	req->recv_func       = (HttpReceiveFunc)   handle_recv;
-	req->add_header_func = (HttpAddHeaderFunc) handle_add_headers;
-	req->close_req_func  = (HttpCloseFunc)     handle_close_request;
+	req->recv_func       = handle_recv;
+	req->add_header_func = handle_add_headers;
+	req->close_req_func  = handle_close_request;
+	req->redirect_func   = handle_redirect;
 
-	http_request_set_conn    (req, c);            /* setup references */
-	http_request_set_timeout (req, 2 * MINUTES);  /* don't wait forever */
-	http_request_set_max_len (req, 65536);        /* don't read forever */
+	gt_http_request_set_conn    (req, c);               /* setup references */
+	gt_http_request_set_proxy   (req, gt_proxy_server); /* maybe use proxy */
+	gt_http_request_set_timeout (req, 2 * MINUTES);     /* don't wait forever */
+	gt_http_request_set_max_len (req, 65536);           /* don't read forever */
 
 	input_add (c->fd, c, INPUT_WRITE,
-	           (InputCallback)http_request_handle, TIMEOUT_DEF);
+	           (InputCallback)gt_http_request_handle, TIMEOUT_DEF);
 
 	return TRUE;
 }
@@ -558,19 +666,19 @@ static void foreach_rand_cache (ds_data_t *key, ds_data_t *value,
 	if (args->now - atime < CACHE_RETRY_TIME)
 		return;
 
-	/* 
-	 * Make sure the cache has a parseable url 
+	/*
+	 * Make sure the cache has a parseable url
 	 *
-	 * TODO: This is ugly, it really should be parsed into a 
+	 * TODO: This is ugly, it really should be parsed into a
 	 * a data structure once instead.
 	 */
 	str = STRDUP (url);
-	ret = parse_web_cache_url (str, &hostname, &path);
+	ret = gt_http_url_parse (str, &hostname, &path);
 	free (str);
 
 	if (!ret)
 	{
-		GT->warn (GT, "bad webcache url \"%s\" from %s/gwebcaches", 
+		GT->warn (GT, "bad webcache url \"%s\" from %s/gwebcaches",
 		          key->data, gift_conf_path ("Gnutella"));
 		return;
 	}
@@ -630,7 +738,7 @@ static BOOL get_random_cache (time_t now, char **r_host_name,
 		return FALSE;
 	}
 
-	ret = parse_web_cache_url (args.url, r_host_name, r_remote_path);
+	ret = gt_http_url_parse (args.url, r_host_name, r_remote_path);
 
 	if (!*r_host_name || !*r_remote_path)
 	{
@@ -649,21 +757,6 @@ static BOOL get_random_cache (time_t now, char **r_host_name,
 	return ret;
 }
 
-static void set_cache_atime (const char *host_name, const char *remote_path, 
-                             time_t atime)
-{
-	char *url;
-	char *field;
-
-	url   = stringf_dup ("http://%s/%s", host_name, remote_path);
-	field = stringf_dup ("%lu", atime);
-
-	file_cache_insert (web_caches, url, field);
-
-	free (url);
-	free (field);
-}
-                               
 static void access_gwebcaches (void)
 {
 	int     len;
@@ -710,14 +803,14 @@ static void access_gwebcaches (void)
 		 * mostly ask for hosts */
 		if (10.0 * rand() / (RAND_MAX + 1.0) < 1.0)
 		{
-			ret = make_request (host_name, remote_path, 
+			ret = make_request (host_name, remote_path,
 			                    "urlfile=1&client=GIFT&version=" GT_VERSION);
 			url_requests++;
 		}
 		else
 #endif
 		{
-			ret = make_request (host_name, remote_path, 
+			ret = make_request (host_name, remote_path,
 			                    "hostfile=1&client=GIFT&version=" GT_VERSION);
 
 			if (ret)
@@ -730,13 +823,13 @@ static void access_gwebcaches (void)
 		{
 			GT->DBGFN (GT, "hitting web cache [total cache hits %u] "
 			           "(cache: http://%s/%s)", cache_hits,
-			           host_name, remote_path);
+			           host_name, STRING_NOTNULL(remote_path));
 
 			cache_hits++;
 			need_sync = TRUE;
 
 			/* reset the atime for the cache */
-			set_cache_atime (host_name, remote_path, now);
+			insert_webcache (host_name, remote_path, now);
 		}
 
 		free (host_name);
@@ -780,7 +873,7 @@ static BOOL webcache_update (void *udata)
 		return FALSE;
 	}
 
-	/* 
+	/*
 	 * next_atime, the absolute next time we allow ourselves to contact the
 	 * caches, gets set when we sucessfully access the caches, and if we
 	 * manage to get some hosts from a cache we access in an exponentially
@@ -799,13 +892,13 @@ void gt_web_cache_update (void)
 	/*
 	 * Don't use GWebCaches if this node is old.
 	 */
-	if (time (NULL) - GT_RELEASE_DATE >= EDAYS * 30 * 6)
+	if (time (NULL) - GT_RELEASE_DATE >= EDAYS * 365)
 	{
 		static BOOL warned = FALSE;
 
 		if (!warned)
 		{
-			GT->DBGFN (GT, "Not updating webcaches: servent %lu days old",
+			GT->DBGFN (GT, "Not updating from webcaches: servent %lu days old",
 			           (time (NULL) - GT_RELEASE_DATE) / EDAYS);
 		}
 
