@@ -1,5 +1,5 @@
 /*
- * $Id: gt_xfer.c,v 1.83 2004/01/01 10:56:41 hipnod Exp $
+ * $Id: gt_xfer.c,v 1.103 2005/01/05 14:08:40 mkern Exp $
  *
  * Copyright (C) 2001-2003 giFT project (gift.sourceforge.net)
  *
@@ -18,59 +18,74 @@
 
 #include "gt_xfer_obj.h"
 #include "gt_xfer.h"
-
 #include "gt_http_client.h"
 #include "gt_http_server.h"
-
-#include "html.h"
-
 #include "gt_share.h"
 #include "gt_share_file.h"
-
-#include "gt_protocol.h"
 #include "gt_packet.h"
-
 #include "gt_node.h"
 #include "gt_node_list.h"
-
 #include "gt_netorg.h"
 #include "gt_connect.h"
+#include "gt_bind.h"
+#include "gt_utils.h"
+
+#include "encoding/url.h"
 
 #include "transfer/source.h"
 
 /******************************************************************************/
 
+/* maximum number of push connections in limbo each remote user */
+#define PUSH_MAX_LIMBO        gt_config_get_int("transfer/push_max_in_limbo=5")
+
+/******************************************************************************/
+
 /* an alternative to carrying the push TTL around in the source url */
-#define PUSH_MAX_TTL            12
+#define PUSH_MAX_TTL              12
 
-/* timeout push requests after this interval */
-#define INDIRECT_TIMEOUT        (4 * MINUTES)
-
-/* minimum interval between pushes */
-#define PUSH_SEND_INTERVAL      (80 * ESECONDS)
+/* maximum time to keep push connections in "limbo" while awaiting giftd
+ * to reissue Chunks */
+#define PUSH_LIMBO_TIMEOUT        (4 * MINUTES)
 
 /* how long to wait for a PUSH reply before timing out in order to free the
  * Chunk */
-#define PUSH_WAIT_INTERVAL      (30 * SECONDS)
+#define PUSH_WAIT_INTERVAL        (30 * SECONDS)
 
-/******************************************************************************/
+/* minimum interval between pushes */
+#define PUSH_MIN_DEFER_TIME       (30 * ESECONDS)
+
+/* maximum amount of time a push will be forced to wait before being sent */
+#define PUSH_MAX_DEFER_TIME       (10 * EMINUTES)
+
+/*****************************************************************************/
 
 /* this stores information about an indirect ("pushed") download */
 typedef struct gt_push_source
 {
 	gt_guid_t  *guid;
 	in_addr_t   ip;
-	in_addr_t   src_ip;       /* whether this push was to a local source */
-	time_t      last_sent;    /* time of last push sent to this source */
-	List       *xfers;        /* xfers for this source */
-	List       *connections;  /* connection for this source */
+	in_addr_t   src_ip;      /* whether this push was to a local source */
+	List       *xfers;       /* xfers for this source */
+	List       *connections; /* connection for this source */
+	time_t      last_sent;   /* time of last push sent to this source */
+	double      defer_time;  /* min time to wait before sending another push
+	                          * doubles every push to PUSH_MAX_DEFER_TIME */
 } GtPushSource;
 
+/*****************************************************************************/
+
 /* Maps guid->{list of unique GtPushSources} */
 static Dataset *gt_push_requests;
 
 /******************************************************************************/
 
+static void push_source_reset_last_sent (GtPushSource *push_src);
+static void push_source_set_last_sent   (gt_guid_t *guid, in_addr_t ip);
+static BOOL push_source_should_send     (gt_guid_t *guid, in_addr_t ip);
+
+/*****************************************************************************/
+
 /*
  * The source URL is stored on disk and could be outdated if the format has
  * changed. This updates it with any changes when we first read it from
@@ -274,6 +289,13 @@ char *gt_localize_request (GtTransfer *xfer, char *s_path, int *authorized)
 static char *index_request (char *request, size_t size,
                             uint32_t index, const char *filename)
 {
+	/*
+	 * The filename may not have ever been set.  Fail the request in
+	 * this case.
+	 */
+	if (!filename || string_isempty (filename))
+		return NULL;
+
 	/*
 	 * Filename is encoded, we don't support sending unecoded requests
 	 * anymore.  NOTE: filename doesnt have leading '/' here, that may change
@@ -345,9 +367,13 @@ static GtPushSource *gt_push_source_new (gt_guid_t *guid, in_addr_t ip,
 	if (!(src = MALLOC (sizeof (GtPushSource))))
 		return NULL;
 
-	src->guid     = gt_guid_dup (guid);
-	src->ip       = ip;
-	src->src_ip   = src_ip;
+	src->guid        = gt_guid_dup (guid);
+	src->ip          = ip;
+	src->src_ip      = src_ip;
+	src->xfers       = NULL;
+	src->connections = NULL;
+
+	push_source_reset_last_sent (src);
 
 	return src;
 }
@@ -440,25 +466,54 @@ void gt_push_source_add (gt_guid_t *guid, in_addr_t ip, in_addr_t src_ip)
 }
 
 /*****************************************************************************/
+/* Timing controls for push requests */
 
-time_t gt_push_source_last_sent_time_get (gt_guid_t *guid, in_addr_t ip)
+static void push_source_reset_last_sent (GtPushSource *push_src)
 {
-	GtPushSource *src;
-
-	if (!(src = push_source_lookup (guid, ip)))
-		return 0;
-
-	return src->last_sent;
+	push_src->last_sent  = gt_uptime ();  /* wrong */
+	push_src->defer_time = 0.0;
 }
 
-void gt_push_source_last_sent_time_set (gt_guid_t *guid, in_addr_t ip, time_t t)
+static void push_source_set_last_sent (gt_guid_t *guid, in_addr_t ip)
 {
 	GtPushSource *src;
 
 	if (!(src = push_source_lookup (guid, ip)))
 		return;
 
-	src->last_sent = t;
+	time (&src->last_sent);
+}
+
+static BOOL push_source_should_send (gt_guid_t *guid, in_addr_t ip)
+{
+	GtPushSource *src;
+	double        deferred;
+	time_t        now;
+
+	time (&now);
+
+	if (!(src = push_source_lookup (guid, ip)))
+		return FALSE;
+
+	deferred = difftime (now, src->last_sent);
+
+	/* randomize the defer time a bit in order to not send pushes for all
+	 * downloads at once */
+	if (deferred < src->defer_time + -10.0 + 20.0 * rand() / (RAND_MAX + 1.0))
+		return FALSE;
+
+	/*
+	 * Double the defer time interval (the minimum time between sent
+	 * pushes for this source).
+	 */
+	src->defer_time *= 2;
+	if (src->defer_time >= PUSH_MAX_DEFER_TIME)
+		src->defer_time = PUSH_MAX_DEFER_TIME;
+
+	if (src->defer_time == 0)
+		src->defer_time = PUSH_MIN_DEFER_TIME;
+
+	return TRUE;
 }
 
 /*****************************************************************************/
@@ -477,21 +532,24 @@ static void flush_inputs (TCPC *c)
 	input_remove_all (c->fd);
 }
 
-static void continue_download (GtTransfer *xfer, TCPC *c)
+static void continue_download (GtPushSource *push_src, GtTransfer *xfer,
+                               TCPC *c)
 {
-	Chunk      *chunk = NULL;
+	Chunk *chunk;
 
-	gt_transfer_unref (NULL, &chunk, &xfer);
-	assert (chunk != NULL);
+	chunk = gt_transfer_get_chunk (xfer);
 
 	/* remove all previous inputs */
 	flush_inputs (c);
 
-	/* HACK: remove the push wait timeout */
-	timer_remove_zero (&xfer->push_wait_timeout);
+	/* HACK: remove the detach timeout */
+	timer_remove_zero (&xfer->detach_timer);
 
-	/* connect the Chunk, TCPC, and GtTransfer */
-	gt_transfer_ref (c, chunk, xfer);
+	/* connect the TCPC and GtTransfer */
+	gt_transfer_set_tcpc (xfer, c);
+
+	/* update the IP and port for placing in Host: */
+	peer_addr (c->fd, &xfer->ip, &xfer->port);
 
 	/* the connection and the chunk have met up */
 	gt_transfer_status (xfer, SOURCE_WAITING, "Received GIV response");
@@ -499,7 +557,7 @@ static void continue_download (GtTransfer *xfer, TCPC *c)
 	if (HTTP_DEBUG)
 		GT->DBGSOCK (GT, c, "Continuing download for %s", xfer->request);
 
-	input_add (c->fd, c, INPUT_WRITE,
+	input_add (c->fd, xfer, INPUT_WRITE,
 	           (InputCallback)gt_http_client_start, TIMEOUT_DEF);
 }
 
@@ -527,7 +585,7 @@ static void store_conn (GtPushSource *src, TCPC *c)
 	flush_inputs (c);
 
 	input_add (c->fd, c, INPUT_READ,
-	           (InputCallback)reset_conn, 4 * MINUTES);
+	           (InputCallback)reset_conn, PUSH_LIMBO_TIMEOUT);
 
 	assert (!list_find (src->connections, c));
 	src->connections = list_prepend (src->connections, c);
@@ -536,7 +594,7 @@ static void store_conn (GtPushSource *src, TCPC *c)
 		GT->DBGSOCK (GT, c, "storing connection");
 }
 
-int gt_push_source_add_conn (gt_guid_t *guid, in_addr_t ip, TCPC *c)
+BOOL gt_push_source_add_conn (gt_guid_t *guid, in_addr_t ip, TCPC *c)
 {
 	GtTransfer    *xfer;
 	GtPushSource  *push_src;
@@ -553,6 +611,28 @@ int gt_push_source_add_conn (gt_guid_t *guid, in_addr_t ip, TCPC *c)
 		return FALSE;
 	}
 
+	/*
+	 * Don't allow too many connections in flight from the same remote user.
+	 */
+	if (list_length (push_src->connections) >= PUSH_MAX_LIMBO)
+	{
+		if (HTTP_DEBUG)
+		{
+			GT->DBGSOCK (GT, c, "too many push connections from %s, closing",
+			             gt_guid_str (guid));
+		}
+
+		tcp_close (c);
+		return FALSE;
+	}
+
+	/*
+	 * Since we now know this push source is alive, reset the push send
+	 * tracking time: in case the connection is lost, we'll resend the push
+	 * right away instead of waiting.
+	 */
+	push_source_reset_last_sent (push_src);
+
 	/*
 	 * Store the connection if there are no GtTransfer requests from
 	 * giFT at the moment.
@@ -566,7 +646,7 @@ int gt_push_source_add_conn (gt_guid_t *guid, in_addr_t ip, TCPC *c)
 	xfer = list_nth_data (push_src->xfers, 0);
 	push_src->xfers = list_remove (push_src->xfers, xfer);
 
-	continue_download (xfer, c);
+	continue_download (push_src, xfer, c);
 	return TRUE;
 }
 
@@ -595,8 +675,8 @@ static void store_xfer (GtPushSource *src, GtTransfer *xfer)
 	src->xfers = list_prepend (src->xfers, xfer);
 }
 
-int gt_push_source_add_xfer (gt_guid_t *guid, in_addr_t ip,
-                             in_addr_t src_ip, GtTransfer *xfer)
+BOOL gt_push_source_add_xfer (gt_guid_t *guid, in_addr_t ip,
+                              in_addr_t src_ip, GtTransfer *xfer)
 {
 	TCPC           *c;
 	GtPushSource   *push_src;
@@ -630,13 +710,13 @@ int gt_push_source_add_xfer (gt_guid_t *guid, in_addr_t ip,
 	c = list_nth_data (push_src->connections, 0);
 	push_src->connections = list_remove (push_src->connections, c);
 
-	continue_download (xfer, c);
+	continue_download (push_src, xfer, c);
 	return TRUE;
 }
 
 /*****************************************************************************/
 
-static int remove_xfer (GtPushSource *src, GtTransfer *xfer)
+static BOOL remove_xfer (GtPushSource *src, GtTransfer *xfer)
 {
 	src->xfers = list_remove (src->xfers, xfer);
 	return FALSE;
@@ -664,7 +744,7 @@ void gt_push_source_remove_xfer (GtTransfer *xfer)
 	dataset_foreach (gt_push_requests, DS_FOREACH(remove_xfer_list), xfer);
 }
 
-static int remove_conn (GtPushSource *src, TCPC *c)
+static BOOL remove_conn (GtPushSource *src, TCPC *c)
 {
 	src->connections = list_remove (src->connections, c);
 	return FALSE;
@@ -688,13 +768,13 @@ void gt_push_source_remove_conn (TCPC *c)
 	dataset_foreach (gt_push_requests, DS_FOREACH(remove_conn_list), c);
 }
 
-static int cleanup_xfer (GtTransfer *xfer, void *udata)
+static BOOL cleanup_xfer (GtTransfer *xfer, void *udata)
 {
 	gt_push_source_remove_xfer (xfer);
 	return TRUE;
 }
 
-static int cleanup_conn (TCPC *c, void *udata)
+static BOOL cleanup_conn (TCPC *c, void *udata)
 {
 	gt_push_source_remove_conn (c);
 	tcp_close (c);
@@ -734,24 +814,72 @@ void gt_push_source_remove (gt_guid_t *guid, in_addr_t ip, in_addr_t src_ip)
 
 /*****************************************************************************/
 
-static BOOL push_wait_timeout (GtTransfer *xfer)
+static BOOL detach_timeout (void *udata)
 {
-	gt_transfer_status (xfer, SOURCE_TIMEOUT, "Timed out");
+	GtTransfer *xfer = udata;
+
+	/* Added this on 2004-12-22 to track observed assertion failure in
+	 * gt_transfer_get_chunk. -- mkern
+	 */
+	if (!xfer->chunk || xfer->chunk->udata != xfer)
+	{
+		GT->DBGFN (GT, "Detach timeout troubles. status = %d, "
+		           "text = %s, xfer->ip = %s, "
+		           "xfer = %p, xfer->chunk->udata = %p, "
+		           "xfer->detach_timer = 0x%X",
+		           xfer->detach_status, xfer->detach_msgtxt,
+		           net_ip_str (xfer->ip), xfer,
+		           xfer->chunk->udata, xfer->detach_timer);
+	}
+
+	/* Sometimes gt_transfer_status will trigger an
+	 * assert (xfer->chunk->udata == xfer) failure in gt_transfer_get_chunk.
+	 * But why? Is xfer already freed? Does it have another chunk and the
+	 * timer was not removed?
+	 */
+	gt_transfer_status (xfer, xfer->detach_status, xfer->detach_msgtxt);
 	gt_transfer_close (xfer, TRUE);
+
 	return FALSE;
 }
 
 /*
- * While we are waiting for a connection from the remote host, we've sent out
- * a push but are not usually actively transferring most of the time. That
- * will keep Chunks occupied that might otherwise be transfering, so timeout
- * those after a short period.
+ * Attach a timer that will "detach" the GtTransfer from the Chunk by
+ * cancelling it, but pretend that the Source is in some other state besides
+ * "cancelled" or "timed out" by changing the status text.
+ *
+ * This is useful to keep a semi-consistent UI in certain situations, such as
+ * sending out push requests, and cancelling requests when the remote side has
+ * queued our request.
  */
-static void set_push_wait_timeout (GtTransfer *xfer)
+static void detach_transfer_in (GtTransfer *xfer, SourceStatus status,
+                                char *status_txt, time_t interval)
 {
-	xfer->push_wait_timeout = timer_add (PUSH_WAIT_INTERVAL,
-	                                     (TimerCallback)push_wait_timeout,
-	                                     xfer);
+	char *msg;
+
+	msg = STRDUP (status_txt);
+
+	gt_transfer_status (xfer, status, msg);
+	xfer->detach_status = status;
+
+	free (xfer->detach_msgtxt);
+	xfer->detach_msgtxt = msg;
+
+	xfer->detach_timer = timer_add (interval,
+	                                (TimerCallback)detach_timeout, xfer);
+}
+
+/*
+ * Detach an GtTransfer from its Chunk.
+ */
+static void detach_transfer (GtTransfer *xfer, SourceStatus status,
+                             char *msgtxt)
+{
+	/*
+	 * Cancelling from p->download_start will cause download_pause() to crash.
+	 * So, the detach must happen from timer context.
+	 */
+	detach_transfer_in (xfer, status, msgtxt, 2 * SECONDS);
 }
 
 /*****************************************************************************/
@@ -760,7 +888,7 @@ static void send_push (GtTransfer *xfer, GtSource *gt, TCPC *server)
 {
 	GtPacket *packet;
 
-	if (!(packet = gt_packet_new (GT_PUSH_REQUEST, PUSH_MAX_TTL, NULL)))
+	if (!(packet = gt_packet_new (GT_MSG_PUSH, PUSH_MAX_TTL, NULL)))
 		return;
 
 	gt_packet_put_ustr   (packet, gt->guid, 16);
@@ -785,17 +913,15 @@ static void send_push (GtTransfer *xfer, GtSource *gt, TCPC *server)
 	 *
 	 * This is really an infelicity of the Chunk system.
 	 */
-	set_push_wait_timeout (xfer);
+	detach_transfer_in (xfer, SOURCE_QUEUED_REMOTE, "Sent PUSH request",
+	                    PUSH_WAIT_INTERVAL);
 
-	/*
-	 * Set the last time we sent a push to now.
-	 */
-	gt_push_source_last_sent_time_set (gt->guid, gt->user_ip, time (NULL));
-	gt_transfer_status (xfer, SOURCE_WAITING, "Sent PUSH request");
+	/* set the last time we sent a push to now */
+	push_source_set_last_sent (gt->guid, gt->user_ip);
 }
 
-static int send_push_to_server (in_addr_t server_ip, in_port_t server_port,
-                                GtTransfer *xfer, GtSource *gt)
+static BOOL send_push_to_server (in_addr_t server_ip, in_port_t server_port,
+                                 GtTransfer *xfer, GtSource *gt)
 {
 	GtNode *server;
 
@@ -825,52 +951,23 @@ static int send_push_to_server (in_addr_t server_ip, in_port_t server_port,
 		return FALSE;
 	}
 	else if (gt_conn_need_connections (GT_NODE_ULTRA) > 0 &&
+	         !server->tried_connect &&
 	         gt_connect (server) >= 0)
 	{
-		/* We tried to connect to the server so we could deliver the
-		 * push request eventually
-		 * NOTE: this doesnt send a push until the next chunk timeout */
+		/*
+		 * We've tried to connect to the server so we could deliver the push
+		 * request eventually NOTE: this doesnt send a push until the next
+		 * chunk timeout.
+		 */
 		return FALSE;
 	}
 
 	return FALSE;
 }
 
-static void detach_transfer (GtTransfer *xfer, Chunk *chunk, 
-                             SourceStatus status, char *status_txt)
-{
-	char *msg; 
-
-	/* duplicate the status text so it can also use the existing
-	 * status data...quite a hack */
-	msg = STRDUP (status_txt);
-
-	gt_transfer_status (xfer, status, msg);
-	free (msg);
-
-	/* 
-	 * Detach the xfer and the chunk. We don't want to return FALSE in
-	 * gnutella_download_start() because that prints a nasty "Protocol
-	 * Error" message to the user, but at the same time for all intents the
-	 * transfer is cancelled here.
-	 */
-	gt_transfer_ref (NULL, NULL, xfer);
-	gt_transfer_ref (NULL, chunk, NULL);
-
-	gt_transfer_close (xfer, TRUE);
-}
-
 static void handle_push_download (Chunk *chunk, GtTransfer *xfer, GtSource *gt)
 {
 	GtNode *server;
-	time_t  last_time;
-	time_t  interval;
-
-	/* this is done in gnutella_download_start now */
-#if 0
-	/* connect the Chunk and the GtTransfer */
-	gt_transfer_ref (NULL, chunk, xfer);
-#endif
 
 	/*
 	 * If this succeeds, we already have a connection to this
@@ -882,27 +979,16 @@ static void handle_push_download (Chunk *chunk, GtTransfer *xfer, GtSource *gt)
 	if (gt_push_source_add_xfer (gt->guid, gt->user_ip, gt->server_ip, xfer))
 		return;
 
-	last_time = gt_push_source_last_sent_time_get (gt->guid, gt->user_ip);
-
-	/*
-	 * The interval we use for sending push requests.
-	 *
-	 * This is offset slightly randomly in order to try to not send the
-	 * requests for all downloads at once.
-	 */
-	interval = PUSH_SEND_INTERVAL + (20.0 * rand() / (RAND_MAX + 1.0));
-
 	/*
 	 * Dont send pushes too often. Maybe should use a global queue instead.
 	 *
 	 * NOTE: we can't free the xfer here because we have stored it.
 	 */
-	if (last_time > 0 && time (NULL) - last_time < interval)
+	if (push_source_should_send (gt->guid, gt->user_ip) == FALSE)
 	{
 		/* don't occupy the Chunk forever */
-		set_push_wait_timeout (xfer);
-
-		gt_transfer_status (xfer, SOURCE_WAITING, "Awaiting connection");
+		detach_transfer_in (xfer, SOURCE_QUEUED_REMOTE, "Awaiting connection",
+		                    PUSH_WAIT_INTERVAL);
 		return;
 	}
 
@@ -925,11 +1011,11 @@ static void handle_push_download (Chunk *chunk, GtTransfer *xfer, GtSource *gt)
 		return;
 	}
 
-	detach_transfer (xfer, chunk, SOURCE_WAITING, "No PUSH route");
+	detach_transfer (xfer, SOURCE_QUEUED_REMOTE, "No PUSH route");
 }
 
-static int set_request (GtTransfer *xfer, Chunk *chunk, Source *source,
-                        GtSource *gt_src)
+static BOOL set_request (GtTransfer *xfer, Chunk *chunk, Source *source,
+                         GtSource *gt_src)
 {
 	char  *request;
 
@@ -945,38 +1031,11 @@ static int set_request (GtTransfer *xfer, Chunk *chunk, Source *source,
 	}
 
 	/* connect the xfer and the chunk */
-	gt_transfer_ref (NULL, chunk, xfer);
+	gt_transfer_set_chunk (xfer, chunk);
 
 	return TRUE;
 }
 
-static int is_firewalled_source (GtSource *gt)
-{
-	if (gt_is_local_ip (gt->user_ip, gt->server_ip))
-		return TRUE;
-
-	if (gt->firewalled)
-		return TRUE;
-
-	return FALSE;
-}
-
-static int self_is_firewalled (void)
-{
-	if (!GT_SELF->firewalled)
-		return FALSE;
-
-	/* 
-	 * Pretend we are not firewalled at the beginning in order
-	 * to possibly get more connections, to prove we are not firewalled. 
-	 */
-	if (gt_uptime () < 10 * EMINUTES)
-		return FALSE;
-
-	/* we are firewalled */
-	return TRUE;
-}
-
 static BOOL should_push (GtSource *gt)
 {
 	TCPC *persistent;
@@ -985,13 +1044,13 @@ static BOOL should_push (GtSource *gt)
 	if (gt_guid_is_empty (gt->guid))
 		return FALSE;
 
-	persistent = gt_http_connection_lookup (gt->user_ip,
-	                                        gt->user_port,
-	                                        GT_TRANSFER_DOWNLOAD);
+	persistent = gt_http_connection_lookup (GT_TRANSFER_DOWNLOAD,
+	                                        gt->user_ip,
+	                                        gt->user_port);
 
 	/* need to close the connection to re-add it to the list, because
 	 * _lookup removes it from the persistent connection list */
-	gt_http_connection_close (persistent, FALSE, GT_TRANSFER_DOWNLOAD);
+	gt_http_connection_close (GT_TRANSFER_DOWNLOAD, persistent, FALSE);
 
 	/* if we already have a connection don't send a push */
 	if (persistent)
@@ -1002,12 +1061,16 @@ static BOOL should_push (GtSource *gt)
 	if (push_source_lookup_conn (gt->guid, gt->user_ip))
 		return TRUE;
 
+	/* send a push if the IP is local */
+	if (gt_is_local_ip (gt->user_ip, gt->server_ip))
+		return TRUE;
+
 	/* don't send a push if we cannot receive incoming connections */
-	if (self_is_firewalled())
+	if (gt_bind_is_firewalled())
 		return FALSE;
 
-	/* we should send a push if they set the firewalled bit */
-	if (is_firewalled_source (gt))
+	/* send a push if they set the firewalled bit */
+	if (gt->firewalled)
 		return TRUE;
 
 	/* the last connection attempt failed, so try a push */
@@ -1055,20 +1118,12 @@ int gnutella_download_start (Protocol *p, Transfer *transfer, Chunk *chunk,
 	gt = source->udata;
 	assert (gt != NULL);
 
-	/* bug's a bit too likely */
-#if 0
-	/* trying to catch a bug */
+	/* giftd should send us only deactivated Chunks */
 	assert (chunk->udata == NULL);
-#endif
 
+	/* free the Source URL and update it with any format changes */
 	replace_url (source, gt);
 
-#if 0
-	GT->dbg (GT, "ip=%s port=%hu guid=%s index=%u filename=%s",
-	         net_ip_str (gt->user_ip), gt->user_port,
-	         gt_guid_str (gt->guid), gt->index, gt->filename);
-#endif
-
 	/* thank you, pretender :) */
 	start = chunk->start + chunk->transmit;
 	stop  = chunk->stop;
@@ -1088,7 +1143,7 @@ int gnutella_download_start (Protocol *p, Transfer *transfer, Chunk *chunk,
 
 	if (download_is_queued (gt))
 	{
-		detach_transfer (xfer, chunk, SOURCE_QUEUED_REMOTE, gt->status_txt);
+		detach_transfer (xfer, SOURCE_QUEUED_REMOTE, gt->status_txt);
 		return TRUE;
 	}
 
@@ -1098,7 +1153,7 @@ int gnutella_download_start (Protocol *p, Transfer *transfer, Chunk *chunk,
 }
 
 void gnutella_download_stop (Protocol *p, Transfer *transfer, Chunk *chunk,
-                             Source *source, int complete)
+                             Source *source, BOOL complete)
 {
 	gt_download_cancel (chunk, NULL);
 }
