@@ -1,5 +1,5 @@
 /*
- * $Id: rx_inflate.h,v 1.2 2003/09/17 17:44:11 hipnod Exp $
+ * $Id: rx_inflate.h,v 1.3 2004/01/31 13:33:17 hipnod Exp $
  *
  * Copyright (C) 2003 giFT project (gift.sourceforge.net)
  *
@@ -19,7 +19,7 @@
 
 /*****************************************************************************/
 
-extern struct rx_layer_ops rx_inflate_ops;
+extern struct rx_layer_ops gt_rx_inflate_ops;
 
 /*****************************************************************************/
 
